@@ -17,7 +17,7 @@
 #define CHANNEL "#lunar-test"
 #define WEBHOOK_PORT 3000
 
-const char *lunabot_version = "0.0.2";
+const char *lunabot_version = "0.0.3";
 
 unsigned int mainloopend;
 int irc_sock;
@@ -58,7 +58,7 @@ char *GetIP(char *hostname) {
 }
 
 // Function to send messages to the IRC channel
-void send_irc_message(const char *message) {
+void SendIrcMessage(const char *message) {
 	char buffer_msg[4096];
 	snprintf(buffer_msg, sizeof(buffer_msg), "PRIVMSG %s :%s\r\n", CHANNEL, message);
 	SSL_write(pSSL, buffer_msg, strlen(buffer_msg));
@@ -159,6 +159,9 @@ static enum MHD_Result WebhookCallback(void *cls, struct MHD_Connection *connect
 		unsigned long *upload_data_size, void **ptr) {
 	static char *json_buffer = NULL;
 	static size_t total_size = 0;
+	static unsigned int cnt = 0;
+
+	fprintf(stderr, "WebhookCallback called: %lu\n", *upload_data_size);
 
 	// On first call, initialize buffer
 	if (*ptr == NULL) {
@@ -172,8 +175,12 @@ static enum MHD_Result WebhookCallback(void *cls, struct MHD_Connection *connect
 		*ptr = json_buffer;
 	}
 
+	// First pass is empty
+	if (*upload_data_size == 0 && cnt == 0)
+		return MHD_YES; // Continue receiving
 	// If there is new data, append it to the buffer
-	if (*upload_data_size > 0) {
+	else if (*upload_data_size > 0) {
+		++cnt;
 		size_t new_size = total_size + *upload_data_size;
 
 		// Reallocate buffer if needed
@@ -192,21 +199,23 @@ static enum MHD_Result WebhookCallback(void *cls, struct MHD_Connection *connect
 		memcpy(json_buffer + total_size, upload_data, *upload_data_size);
 		total_size += *upload_data_size;
 		json_buffer[total_size] = '\0'; // Null-terminate
-		printf("!!%s!!\n", json_buffer);
+		fprintf(stderr, "!!%s!!\n", json_buffer);
 
 		*upload_data_size = 0;
 		return MHD_YES; // Continue receiving
 	}
 	// If we have all data, process JSON
-	else if (strlen(json_buffer)) {
-		printf("Received full webhook JSON: %s\n", json_buffer); // Debugging
+	else if (*upload_data_size == 0 && cnt >= 1) {
+		fprintf(stderr, "!!Received full webhook JSON: %s!!\n", json_buffer); // Debugging
+
+		cnt = 0;
 
 		json_t *root;
 		json_error_t error;
 		root = json_loads(json_buffer, 0, &error);
 
 		if (!root) {
-			fprintf(stderr, "JSON parsing error: %s\n", error.text);
+			fprintf(stderr, "!!JSON parsing error: %s!!\n", error.text);
 		} else {
 			json_t *action = json_object_get(root, "action");
 			json_t *pr = json_object_get(root, "pull_request");
@@ -220,12 +229,11 @@ static enum MHD_Result WebhookCallback(void *cls, struct MHD_Connection *connect
 					if (json_is_string(title) && json_is_string(user) && json_is_string(url)) {
 						char message[512];
 						snprintf(message, sizeof(message), 
-								 "New PR: %s by %s - %s", 
+								 "New PR: '%s' from %s - %s", 
 								 json_string_value(title), 
 								 json_string_value(user), 
 								 json_string_value(url));
-
-						send_irc_message(message);
+						SendIrcMessage(message);
 					}
 				}
 				else if (strcmp(json_string_value(action), "closed") == 0) {
@@ -235,16 +243,16 @@ static enum MHD_Result WebhookCallback(void *cls, struct MHD_Connection *connect
 					if (json_is_string(title) && json_is_string(user) && json_is_string(url)) {
 						char message[512];
 						snprintf(message, sizeof(message),
-							"Closed PR: '%s' by %s - %s",
+							"Closed PR: '%s' from %s - %s",
 							json_string_value(title),
 							json_string_value(user),
 							json_string_value(url));
-
-						printf("!!%s!!\n", message);
-						send_irc_message(message);
+						SendIrcMessage(message);
 					}
 				}
 			}
+			else
+				SendIrcMessage("Got webhook data without a conditional branch for it!");
 			json_decref(root);
 		}
 	}
@@ -261,94 +269,22 @@ static enum MHD_Result WebhookCallback(void *cls, struct MHD_Connection *connect
 	return ret;
 }
 
-static enum MHD_Result WebhookCallback_orig(void *cls, struct MHD_Connection *connection, 
-							const char *url, const char *method, 
-							const char *version, const char *upload_data,
-							unsigned long *upload_data_size, void **ptr) {
-	static int aptr;
-	if (&aptr != *ptr) {
-		*ptr = &aptr;
-		return MHD_YES;
-	}
-	if (*upload_data_size == 0) return MHD_NO;
-
-	// Parse JSON data
-	json_t *root;
-	json_error_t error;
-	root = json_loads(upload_data, 0, &error);
-	printf("!!!!%s!!!!\n", upload_data);
-
-	if (!root) {
-		printf("JSON error: %s\n", error.text);
-		return MHD_NO;
-	}
-
-	json_t *action = json_object_get(root, "action");
-	json_t *pr = json_object_get(root, "pull_request");
-
-	if (json_is_string(action) && json_is_object(pr)) {
-		if (strcmp(json_string_value(action), "opened") == 0) {
-			json_t *title = json_object_get(pr, "title");
-			json_t *user = json_object_get(json_object_get(pr, "user"), "login");
-			json_t *url = json_object_get(pr, "html_url");
-
-			if (json_is_string(title) && json_is_string(user) && json_is_string(url)) {
-				char message[512];
-				snprintf(message, sizeof(message), 
-						 "New PR: '%s' by %s - %s", 
-						 json_string_value(title), 
-						 json_string_value(user), 
-						 json_string_value(url));
-
-				printf("!!%s!!\n", message);
-				send_irc_message(message);
-			}
-		}
-		else if (strcmp(json_string_value(action), "closed") == 0) {
-			json_t *title = json_object_get(pr, "title");
-			json_t *user = json_object_get(json_object_get(pr, "user"), "login");
-			json_t *url = json_object_get(pr, "html_url");
-			if (json_is_string(title) && json_is_string(user) && json_is_string(url)) {
-				char message[512];
-				snprintf(message, sizeof(message),
-					"Closed PR: '%s' by %s - %s",
-					json_string_value(title),
-					json_string_value(user),
-					json_string_value(url));
-
-				printf("!!%s!!\n", message);
-				send_irc_message(message);
-			}
-		}
-	}
-
-	json_decref(root);
-
-	// Send response
-	struct MHD_Response *response = MHD_create_response_from_buffer(16, "OK", MHD_RESPMEM_PERSISTENT);
-	int ret = MHD_queue_response(connection, MHD_HTTP_OK, response);
-	MHD_destroy_response(response);
-
-	return ret;
-}
-
 // Webhook server thread
 void *WebhookServerStart(void *arg) {
 	struct MHD_Daemon *daemon;
 	daemon = MHD_start_daemon(MHD_USE_SELECT_INTERNALLY, WEBHOOK_PORT, NULL, NULL,
 							  &WebhookCallback, NULL, MHD_OPTION_END);
 	if (!daemon) {
-		printf("lunabot::WebhookServerStart(): Failed to start HTTP server\n");
+		fprintf(stderr, "lunabot::WebhookServerStart(): Failed to start HTTP server\n");
 		exit(1);
 	}
 
-	printf("  Webhook server running on port %d...\n", WEBHOOK_PORT);
+	fprintf(stderr, "!!Webhook server running on port %d!!\n", WEBHOOK_PORT);
 
-	while (1)
+	while (!mainloopend)
 		sleep(5);
 
-//	getchar();
-//	MHD_stop_daemon(daemon);
+	MHD_stop_daemon(daemon);
 	return NULL;
 }
 
@@ -383,7 +319,7 @@ int main() {
 		if (strncmp(buffer_line, "exit", 4) == 0 || strncmp(buffer_line, "quit", 4) == 0 ||
 		  strncmp(buffer_line, "qw", 2) == 0)
 			mainloopend = 1;
-		else if (strlen(buffer_line) > 0) { // Send to server, this is a raw message!
+		else if (strlen(buffer_line) > 0 && *buffer_line != '\n') { // Send to server, this is a raw message!
 			if (buffer_line[strlen(buffer_line)-1] == '\n')
 				buffer_line[strlen(buffer_line)-1] = '\0';
 
